
void gifLZPatternTable::output(unsigned n) const
{
      _packer.append(n);
}

void gifLZPatternTable::output(const patternCell *c) const
{
      output(c - _table);
}

void gifLZPatternTable::addCell(unsigned char v, patternCell *lastPattern)
{
      patternCell *last = _last;

      if (last >= _tableEnd) {

	    last = _table;
	    for(unsigned i = 256; i; i--) {
		  last -> _used = 0;
		  last -> _nextWithSameHashValue = NIL;
		  last++;
	    }
	    last += 2; // Clear code and End Of Information code
	    output(lastPattern);
	    output(256);

	    _codeSize = 9;
	    _maxCode = 1 << _codeSize;

      } else {

	    unsigned currentCode = last - _table;
	    if (currentCode >= _maxCode) {

		  assert(_codeSize <= 12);
		  _codeSize++;
		  _maxCode = 1 << _codeSize;
		  output(lastPattern);
		  _packer.incrementCodeSize();
	    } else
		  output(lastPattern);

	    last -> _char = v;
	    last -> _used = 0;
	    last -> _nextWithSameHashValue = NIL;

	    last++;
      }

      _current = _table + v;
      _last = last;
}

gifLZPatternTable::gifLZPatternTable(gifLZPacker *p, unsigned initialValue)
:_packer(*p), _tableEnd(_table + tableSize), _codeSize(9), _maxCode(1 << _codeSize)
{
      patternCell *last = _table;

      for(unsigned i = 0; i <= 255; i++, last++) {
	    last -> _char = i;
	    last -> _used = 0;
	    last -> _nextWithSameHashValue = NIL;
      };

      _last = last + 2; // Clear code and End Of Information code
      _current = _table + initialValue;

      output(256);
}

gifLZPatternTable::~gifLZPatternTable()
{
      output(_current);
      output(257);
}

void gifLZPatternTable::update(unsigned char v)
{
      unsigned hash = ((v & 0xf0) >> 4) | v & 0xf;
      unsigned mask = 1 << hash;

      patternCell **next = _current -> _next + hash;
      patternCell *lastPattern = _current;

      if (_current -> _used & mask) {

	    for(patternCell *q = *next;;) {
		  if (q -> _char == v) {
			_current = q;
			break;
		  } else if (q -> _nextWithSameHashValue) {
			q = q -> _nextWithSameHashValue;
		  } else {
			q -> _nextWithSameHashValue = _last;
			addCell(v, lastPattern);
			break;
		  }
	    }

      } else {
		  
	    // No cell with this hash value
		  
	    _current -> _used |= mask;
	    *next = _last;
	    addCell(v, lastPattern);
      }
}

