//
// Written by Ch. Tronche (http://tronche.lri.fr:8000/)
// Copyright by the author. This is unmaintained, no-warranty free software. 
// Please use freely. It is appreciated (but not mandatory) to acknowledge
// the author's contribution. Thank you.
// Started on Mon Mar  3 10:49:36 1997
//

#include <standardInclude.h>

#include "gifColorTable.h"
#include "oGifStream.h"

static const globalColorTablePresent = 0x80;

static int colorResolution(unsigned n)
{
      return (n & 0x7) << 4;
}

static int colorTableSize(unsigned n)
{
      int v = 0;
      switch(n) {
	  case 0:
	  case 2:
	    break;
	  case 4:  v = 1; break;
	  case 8:  v = 2; break;
	  case 16: v = 3; break;
	  case 32: v = 4; break;
	  case 64: v = 5; break;
	  case 128: v = 6; break;
	  case 256: v = 7; break;
	  default:
	    assert(!"inconsistent color table size");
      }
      return v;
}

oGifStream::oGifStream(ostream *physical, unsigned screenWidth, unsigned screenHeight, const gifColorTable *globalColorTable,
		       unsigned backgroundColor = 0, unsigned pixelAspectRatio = 0, unsigned colorRes = 0x7, 
		       bool globalColorTableSorted = false)
:_screenWidth(screenWidth), _screenHeight(screenHeight), _physical(*physical)
{
      write("GIF89a", 6);
      storeUnsigned16(screenWidth);
      storeUnsigned16(screenHeight);
      unsigned char logicalScreenDescriptor = 0;
      if (globalColorTable)
	    logicalScreenDescriptor |= globalColorTablePresent | colorTableSize(globalColorTable -> entryNumber());
      logicalScreenDescriptor |= colorResolution(colorRes);
      put(logicalScreenDescriptor);
      put(backgroundColor);
      put(pixelAspectRatio);
      if (globalColorTable)
	    write(static_cast<const char *>((*globalColorTable)[0]), globalColorTable -> size());
}
