#include <iostream.h>
#include <error++/error.h>

#include "videoError.h"
#include "videoNode.h"
#include "videoPath.h"
#include "videoServer.h"
#include "videoServer.h"

videoNode::videoNode(videoServer *s, int type, int kind, int number)
{
      _server = s;
      _serverHandle = *s;
      _node = vlGetNode(_serverHandle, type, kind, number);
      if (_node < 0) cerr << "videoNode::videoNode: " << videoError << stop;
}

videoServer *videoNode::server()
{
      return _server;
}

void videoNode::getXYControl(VLPath path, VLControlType ctrl, int *x, int *y) const
{
      VLControlValue ctrlVal;
      if (vlGetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::getXYControl" << videoError << stop;
      *x = ctrlVal.xyVal.x;
      *y = ctrlVal.xyVal.y;
}

void videoNode::setXYControl(VLPath path, VLControlType ctrl, int x, int y) const
{
      VLControlValue ctrlVal;
      ctrlVal.xyVal.x = x;
      ctrlVal.xyVal.y = y;
      if (vlSetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::setXYControl(" << x << ", " << y << "): " << videoError << stop;
}

void videoNode::getSize(const videoPath &path, unsigned *width, unsigned *height) const
{
      int w, h;
      getXYControl(path, VL_SIZE, &w, &h);
      *width = w;
      *height = h;
}

void videoNode::setSize(const videoPath &path, int width, int height)
{
      setXYControl(path, VL_SIZE, width, height);
}

void videoNode::setIntControl(VLPath path, VLControlType ctrl, int value) const
{
      VLControlValue ctrlVal;
      ctrlVal.intVal = value;
      if (vlSetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::setIntControl(" << ctrl  << "[0x" << hex << ctrl << dec << "], " << value << "): " << videoError << stop;
}

void videoNode::setFractControl(VLPath path, VLControlType ctrl, int numerator, int denominator)
{
      VLControlValue ctrlVal;
      ctrlVal.fractVal.numerator = numerator;
      ctrlVal.fractVal.denominator = denominator;
      if (vlSetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::setFractControl(" << ctrl << ',' << numerator << ',' << denominator << "): " << videoError << stop;
}

int videoNode::getIntControl(VLPath path, VLControlType ctrl) const
{
      VLControlValue ctrlVal;
      if (vlGetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::getIntControl: " << videoError << stop;
      return ctrlVal.intVal;
}

void videoNode::getFractControl(VLPath path, VLControlType ctrl, int *numerator, int *denominator)
{
      VLControlValue ctrlVal;
      if (vlGetControl(_serverHandle, path, _node, ctrl, &ctrlVal) < 0)
	    cerr << "videoNode::getFractControl: " << videoError << stop;
      *numerator = ctrlVal.fractVal.numerator;
      *denominator = ctrlVal.fractVal.denominator;
}
